import psycopg2
from psycopg2.extras import DictCursor as _cursor_factory

from conf import DB_HOST, DB_USER, DB_PASSWD, DB_NAME
from speaker import Speaker, YELLOW, RED, init_speaker
from eccez import QueryError

def _connectStr():
    return DB_HOST                                  \
        and                                         \
        "dbname=%s user=%s host=%s password=%s"     \
            % (DB_NAME,DB_USER,DB_HOST,DB_PASSWD)   \
        or                                          \
        "dbname=%s user=%s password=%s"             \
            % (DB_NAME,DB_USER,DB_PASSWD)       


class Psygres(Speaker):

    _db_initialized= False
    trans_level= 0
    pg_conn= None
    pg_cursor= None

    @staticmethod
    def _connect(pg_conn= None):

        if Psygres._db_initialized:
            Speaker.log_error("Psygres already connected.")
            return

        if pg_conn:
            Speaker.log_info("Initializing Psygres with own db connection.")
            Psygres.pg_conn= pg_conn
            try:
                # Zope WorkAround
                Psygres.pg_cursor= pg_conn.getcursor()
            except:
                Psygres.pg_cursor= pg_conn.cursor()
        else:
            Psygres.pg_conn= psycopg2.connect(_connectStr())
            Psygres.pg_cursor= Psygres.pg_conn.cursor(
                    cursor_factory= _cursor_factory)

        Psygres._db_initialized= True

        return

    @staticmethod
    def beginTrans():
        Psygres.trans_level+= 1
        Speaker.log_debug(
                'Incremented transaction level: %s', 
                YELLOW(Psygres.trans_level))

    @staticmethod
    def rollback():
        if not Psygres.trans_level:
            Speaker.log_debug(
                    "Nothing to rollback: "
                    "nested rollback?")
            return

        Psygres.pg_conn.rollback()
        Psygres.trans_level= 0

        return

    @staticmethod
    def commit():
        lvl= Psygres.trans_level
        if not lvl:
            Speaker.log_error(
                    "Nothing to commit: "
                    "nested commit?")
            return

        if lvl== 1:
            Psygres.pg_conn.commit()
            Psygres.trans_level-= 1
            Speaker.log_debug(
                    "Queries committed.")
            return

        Psygres.trans_level-= 1
        Speaker.log_debug(
                "Decremented transaction: now %s",
                YELLOW(Psygres.trans_level))
        return

    @staticmethod
    def mogrify(s, d):
        return Psygres.pg_cursor.mogrify(s, d)

    @staticmethod
    def lastrowid():
        return Psygres.pg_cursor.lastrowid


    @staticmethod
    def _get_query(s):

        Speaker.log_info("QSQL- %s", s)
        cursor= Psygres.pg_cursor

        try:
            cursor.execute(s)
            res= cursor.fetchall()

        except Exception, es:
            Psygres.rollback()
            Speaker.log_raise(
                    "Unable to execute query: %s" % 
                    RED(es), QueryError)

        dict_list= []
        for r in res:
            d={}
            d.update(r)
            dict_list.append(d)
        return dict_list

    @staticmethod
    def _quiet_query(s):

        Speaker.log_info("QSQL- %s", s)
        cursor= Psygres.pg_cursor

        try:
            cursor.execute(s)
            if not Psygres.trans_level:
                Psygres.pg_conn.commit()
            else:
                Speaker.log_debug(
                        "Inside Trans: query not committed.")

        except Exception, es:
            dbda.rollback()
            Speaker.log_raise(
                    "Unable to execute query: %s" % 
                    RED(es), QueryError)

    @staticmethod
    def oc_query(s):
        """ One Commit Query: no returns."""
        Psygres._quiet_query(s)

    @staticmethod
    def mr_query(s):
        """ Multiple Records Query: return a dict list."""
        return Psygres._get_query(s)

    @staticmethod
    def or_query(s):
        """ One Record Query: returns a dict."""

        res= Psygres.mr_query(s)
        
        if len(res)<> 1:
            Speaker.log_raise(
                    "or_query() returned %s records." % 
                    RED(recs), QueryError)

        return res[0]

    @staticmethod
    def ov_query(s):
        """ One Value Query: returns one only value."""
        try:
            res= Psygres.or_query(s)
        except:
            Speaker.log_raise(
                    "ov_query() returned %s records." % 
                    RED(recs), QueryError)

        res= res[0]
        if len(res)<> 1:
            Speaker.log_raise(
                    "ov_query() returned %s values." % 
                    RED(recs), QueryError)
        return res


def init_dbda(pg_conn= None):

    if Psygres._db_initialized:
        Speaker.log_error("Dbda already initialized.")
        return

    if not Speaker._spkr_initialized:
        init_speaker()

    Psygres._connect(pg_conn)
